@@ -173,6 +173,7 @@
 #define PCI_CHIP_RV410_564F 0x564F
 #define PCI_CHIP_RV410_5652 0x5652
 #define PCI_CHIP_RV410_5653 0x5653
+#define PCI_CHIP_RV410_5657 0x5657
 #define PCI_CHIP_MACH64VT 0x5654
 #define PCI_CHIP_MACH64VU 0x5655
 #define PCI_CHIP_MACH64VV 0x5656
@@ -195,7 +196,6 @@
 #define PCI_CHIP_RV370_5B60 0x5B60
 #define PCI_CHIP_RV370_5B62 0x5B62
 #define PCI_CHIP_RV370_5B63 0x5B63
-#define PCI_CHIP_RV370_5657 0x5657
 #define PCI_CHIP_RV370_5B64 0x5B64
 #define PCI_CHIP_RV370_5B65 0x5B65
 #define PCI_CHIP_RV280_5C61 0x5C61
@@ -281,9 +281,9 @@
 #define PCI_CHIP_RV530_71D6 0x71D6
 #define PCI_CHIP_RV530_71DA 0x71DA
 #define PCI_CHIP_RV530_71DE 0x71DE
-#define PCI_CHIP_RV530_7200 0x7200
-#define PCI_CHIP_RV530_7210 0x7210
-#define PCI_CHIP_RV530_7211 0x7211
+#define PCI_CHIP_RV515_7200 0x7200
+#define PCI_CHIP_RV515_7210 0x7210
+#define PCI_CHIP_RV515_7211 0x7211
 #define PCI_CHIP_R580_7240 0x7240
 #define PCI_CHIP_R580_7243 0x7243
 #define PCI_CHIP_R580_7244 0x7244
@@ -315,6 +315,9 @@
 #define PCI_CHIP_RS350_7835 0x7835
 #define PCI_CHIP_RS690_791E 0x791E
 #define PCI_CHIP_RS690_791F 0x791F
+#define PCI_CHIP_RS600_793F 0x793F
+#define PCI_CHIP_RS600_7941 0x7941
+#define PCI_CHIP_RS600_7942 0x7942
 #define PCI_CHIP_RS740_796C 0x796C
 #define PCI_CHIP_RS740_796D 0x796D
 #define PCI_CHIP_RS740_796E 0x796E
@@ -356,3 +359,22 @@
 #define PCI_CHIP_RV630_958C 0x958C
 #define PCI_CHIP_RV630_958D 0x958D
 #define PCI_CHIP_RV630_958E 0x958E
+#define PCI_CHIP_RV620_95C0 0x95C0
+#define PCI_CHIP_RV620_95C5 0x95C5
+#define PCI_CHIP_RV620_95C7 0x95C7
+#define PCI_CHIP_RV620_95C2 0x95C2
+#define PCI_CHIP_RV620_95C4 0x95C4
+#define PCI_CHIP_RV620_95CD 0x95CD
+#define PCI_CHIP_RV620_95CE 0x95CE
+#define PCI_CHIP_RV620_95CF 0x95CF
+#define PCI_CHIP_RV635_9590 0x9590
+#define PCI_CHIP_RV635_9596 0x9596
+#define PCI_CHIP_RV635_9597 0x9597
+#define PCI_CHIP_RV635_9598 0x9598
+#define PCI_CHIP_RV635_9599 0x9599
+#define PCI_CHIP_RV635_9591 0x9591
+#define PCI_CHIP_RV635_9593 0x9593
+#define PCI_CHIP_RS780_9610 0x9610
+#define PCI_CHIP_RS780_9611 0x9611
+#define PCI_CHIP_RS780_9612 0x9612
+#define PCI_CHIP_RS780_9613 0x9613
