@@ -46,7 +46,7 @@
 struct blendinfo {
 	Bool dst_alpha;
 	Bool src_alpha;
-	CARD32 blend_cntl;
+	uint32_t blend_cntl;
 };
 
 /* The first part of blend_cntl corresponds to Fa from the render "protocol"
@@ -166,8 +166,8 @@ static CARD32 RADEONDstFormats[] = {
     0
 };
 
-static CARD32
-RadeonGetTextureFormat(CARD32 format)
+static uint32_t
+RadeonGetTextureFormat(uint32_t format)
 {
     switch (format) {
     case PICT_a8r8g8b8:
@@ -187,8 +187,8 @@ RadeonGetTextureFormat(CARD32 format)
     }
 }
 
-static CARD32
-RadeonGetColorFormat(CARD32 format)
+static uint32_t
+RadeonGetColorFormat(uint32_t format)
 {
     switch (format) {
     case PICT_a8r8g8b8:
@@ -207,10 +207,10 @@ RadeonGetColorFormat(CARD32 format)
 /* Returns a RADEON_RB3D_BLENDCNTL value, or 0 if the operation is not
  * supported
  */
-static CARD32
-RadeonGetBlendCntl(CARD8 op, CARD32 dstFormat)
+static uint32_t
+RadeonGetBlendCntl(uint8_t op, uint32_t dstFormat)
 {
-    CARD32 blend_cntl;
+    uint32_t blend_cntl;
 
     if (op >= RadeonOpMax || RadeonBlendOp[op].blend_cntl == 0)
 	return 0;
@@ -218,7 +218,7 @@ RadeonGetBlendCntl(CARD8 op, CARD32 dstFormat)
     blend_cntl = RadeonBlendOp[op].blend_cntl;
 	
     if (RadeonBlendOp[op].dst_alpha && !PICT_FORMAT_A(dstFormat)) {
-	CARD32 srcblend = blend_cntl & RADEON_SRC_BLEND_MASK;
+	uint32_t srcblend = blend_cntl & RADEON_SRC_BLEND_MASK;
 
 	/* If there's no destination alpha channel, we need to wire the blending
 	 * to treat the alpha channel as always 1.
@@ -235,11 +235,11 @@ RadeonGetBlendCntl(CARD8 op, CARD32 dstFormat)
     return blend_cntl;
 }
 
-static __inline__ CARD32 F_TO_DW(float val)
+static __inline__ uint32_t F_TO_DW(float val)
 {
     union {
 	float f;
-	CARD32 l;
+	uint32_t l;
     } tmp;
     tmp.f = val;
     return tmp.l;
@@ -250,10 +250,17 @@ static __inline__ int
 ATILog2(int val)
 {
 	int bits;
-
+#if (defined __i386__ || defined __x86_64__) && (defined __GNUC__)
+	__asm volatile("bsrl	%1, %0"
+		: "=r" (bits)
+		: "c" (val)
+	);
+	return bits;
+#else
 	for (bits = 0; val != 0; val >>= 1, ++bits)
 		;
 	return bits - 1;
+#endif
 }
 
 static void
@@ -317,7 +324,7 @@ static Bool RADEONSetupRenderByteswap(ScrnInfoPtr pScrn, int tex_bytepp)
 {
     RADEONInfoPtr info = RADEONPTR(pScrn);
     unsigned char *RADEONMMIO = info->MMIO;
-    CARD32 swapper = info->ModeReg.surface_cntl;
+    uint32_t swapper = info->ModeReg->surface_cntl;
 
     swapper &= ~(RADEON_NONSURF_AP0_SWP_16BPP | RADEON_NONSURF_AP1_SWP_16BPP |
 		 RADEON_NONSURF_AP0_SWP_32BPP | RADEON_NONSURF_AP1_SWP_32BPP);
@@ -345,7 +352,7 @@ static void RADEONRestoreByteswap(RADEONInfoPtr info)
 {
     unsigned char *RADEONMMIO = info->MMIO;
 
-    OUTREG(RADEON_SURFACE_CNTL, info->ModeReg.surface_cntl);
+    OUTREG(RADEON_SURFACE_CNTL, info->ModeReg->surface_cntl);
 }
 #endif	/* X_BYTE_ORDER == X_BIG_ENDIAN */
 
@@ -373,35 +380,38 @@ static void RADEONRestoreByteswap(RADEONInfoPtr info)
 
 static Bool FUNC_NAME(R100SetupTexture)(
 	ScrnInfoPtr pScrn,
-	CARD32 format,
-	CARD8 *src,
+	uint32_t format,
+	uint8_t *src,
 	int src_pitch,
 	unsigned int width,
 	unsigned int height,
 	int flags)
 {
     RADEONInfoPtr info = RADEONPTR(pScrn);
-    CARD8 *dst;
-    CARD32 tex_size = 0, txformat;
+    uint8_t *dst;
+    uint32_t tex_size = 0, txformat;
     int dst_pitch, offset, size, tex_bytepp;
 #ifdef ACCEL_CP
-    CARD32 buf_pitch, dst_pitch_off;
+    uint32_t buf_pitch, dst_pitch_off;
     int x, y;
     unsigned int hpass;
-    CARD8 *tmp_dst;
+    uint8_t *tmp_dst;
 #endif
     ACCEL_PREAMBLE();
 
-    /* render repeat is broken - fix in stable tree by falling back */
-    if (flags & XAA_RENDER_REPEAT)
-	return FALSE;
-
-    if ((width > 2048) || (height > 2048))
+    if ((width > 2047) || (height > 2047))
 	return FALSE;
 
     txformat = RadeonGetTextureFormat(format);
     tex_bytepp = PICT_FORMAT_BPP(format) >> 3;
 
+    dst_pitch = (width * tex_bytepp + 63) & ~63;
+    size = dst_pitch * height;
+
+    if ((flags & XAA_RENDER_REPEAT) && (height != 1) &&
+	(((width * tex_bytepp + 31) & ~31) != dst_pitch))
+	return FALSE;
+
 #ifndef ACCEL_CP
 
 #if X_BYTE_ORDER == X_BIG_ENDIAN
@@ -414,9 +424,6 @@ static Bool FUNC_NAME(R100SetupTexture)(
 
 #endif
 
-    dst_pitch = (width * tex_bytepp + 63) & ~63;
-    size = dst_pitch * height;
-
     if (!AllocateLinear(pScrn, size))
 	return FALSE;
 
@@ -424,12 +431,12 @@ static Bool FUNC_NAME(R100SetupTexture)(
 	txformat |= ATILog2(width) << RADEON_TXFORMAT_WIDTH_SHIFT;
 	txformat |= ATILog2(height) << RADEON_TXFORMAT_HEIGHT_SHIFT;
     } else {
-	tex_size = ((height - 1) << 16) | (width - 1);
+	tex_size = (height << 16) | width;
 	txformat |= RADEON_TXFORMAT_NON_POWER2;
     }
 
     offset = info->RenderTex->offset * pScrn->bitsPerPixel / 8;
-    dst = (CARD8*)(info->FB + offset);
+    dst = (uint8_t*)(info->FB + offset);
 
     /* Upload texture to card. */
 
@@ -500,7 +507,7 @@ FUNC_NAME(R100SetupForCPUToScreenAlphaTexture) (
 ) 
 {
     RADEONInfoPtr info = RADEONPTR(pScrn);
-    CARD32 colorformat, srccolor, blend_cntl;
+    uint32_t colorformat, srccolor, blend_cntl;
     ACCEL_PREAMBLE();
 
     blend_cntl = RadeonGetBlendCntl(op, dstFormat);
@@ -551,7 +558,7 @@ FUNC_NAME(R100SetupForCPUToScreenTexture) (
 )
 {
     RADEONInfoPtr info = RADEONPTR(pScrn);
-    CARD32 colorformat, blend_cntl;
+    uint32_t colorformat, blend_cntl;
     ACCEL_PREAMBLE();
 
     blend_cntl = RadeonGetBlendCntl(op, dstFormat);
@@ -598,7 +605,7 @@ FUNC_NAME(R100SubsequentCPUToScreenTexture) (
 {
     RADEONInfoPtr  info       = RADEONPTR(pScrn);
     int byteshift;
-    CARD32 fboffset;
+    uint32_t fboffset;
     float l, t, r, b, fl, fr, ft, fb;
 
     ACCEL_PREAMBLE();
@@ -710,35 +717,38 @@ FUNC_NAME(R100SubsequentCPUToScreenTexture) (
 
 static Bool FUNC_NAME(R200SetupTexture)(
 	ScrnInfoPtr pScrn,
-	CARD32 format,
-	CARD8 *src,
+	uint32_t format,
+	uint8_t *src,
 	int src_pitch,
 	unsigned int width,
 	unsigned int height,
 	int flags)
 {
     RADEONInfoPtr info = RADEONPTR(pScrn);
-    CARD8 *dst;
-    CARD32 tex_size = 0, txformat;
+    uint8_t *dst;
+    uint32_t tex_size = 0, txformat;
     int dst_pitch, offset, size, tex_bytepp;
 #ifdef ACCEL_CP
-    CARD32 buf_pitch, dst_pitch_off;
+    uint32_t buf_pitch, dst_pitch_off;
     int x, y;
     unsigned int hpass;
-    CARD8 *tmp_dst;
+    uint8_t *tmp_dst;
 #endif
     ACCEL_PREAMBLE();
 
-    /* render repeat is broken - fix in stable tree by falling back */
-    if (flags & XAA_RENDER_REPEAT)
-	return FALSE;
-
     if ((width > 2048) || (height > 2048))
 	return FALSE;
 
     txformat = RadeonGetTextureFormat(format);
     tex_bytepp = PICT_FORMAT_BPP(format) >> 3;
 
+    dst_pitch = (width * tex_bytepp + 63) & ~63;
+    size = dst_pitch * height;
+
+    if ((flags & XAA_RENDER_REPEAT) && (height != 1) &&
+	(((width * tex_bytepp + 31) & ~31) != dst_pitch))
+	return FALSE;
+
 #ifndef ACCEL_CP
 
 #if X_BYTE_ORDER == X_BIG_ENDIAN
@@ -751,9 +761,6 @@ static Bool FUNC_NAME(R200SetupTexture)(
 
 #endif
 
-    dst_pitch = (width * tex_bytepp + 63) & ~63;
-    size = dst_pitch * height;
-
     if (!AllocateLinear(pScrn, size))
 	return FALSE;
 
@@ -765,8 +772,11 @@ static Bool FUNC_NAME(R200SetupTexture)(
 	txformat |= RADEON_TXFORMAT_NON_POWER2;
     }
 
+    info->texW[0] = width;
+    info->texH[0] = height;
+
     offset = info->RenderTex->offset * pScrn->bitsPerPixel / 8;
-    dst = (CARD8*)(info->FB + offset);
+    dst = (uint8_t*)(info->FB + offset);
 
     /* Upload texture to card. */
 
@@ -838,7 +848,7 @@ FUNC_NAME(R200SetupForCPUToScreenAlphaTexture) (
 ) 
 {
     RADEONInfoPtr  info = RADEONPTR(pScrn);
-    CARD32 colorformat, srccolor, blend_cntl;
+    uint32_t colorformat, srccolor, blend_cntl;
     ACCEL_PREAMBLE();
 
     blend_cntl = RadeonGetBlendCntl(op, dstFormat);
@@ -890,7 +900,7 @@ FUNC_NAME(R200SetupForCPUToScreenTexture) (
 )
 {
     RADEONInfoPtr  info       = RADEONPTR(pScrn);
-    CARD32 colorformat, blend_cntl;
+    uint32_t colorformat, blend_cntl;
     ACCEL_PREAMBLE();
 
     blend_cntl = RadeonGetBlendCntl(op, dstFormat);
@@ -938,7 +948,7 @@ FUNC_NAME(R200SubsequentCPUToScreenTexture) (
 {
     RADEONInfoPtr  info       = RADEONPTR(pScrn);
     int byteshift;
-    CARD32 fboffset;
+    uint32_t fboffset;
     float l, t, r, b, fl, fr, ft, fb;
     ACCEL_PREAMBLE();
 
@@ -964,10 +974,10 @@ FUNC_NAME(R200SubsequentCPUToScreenTexture) (
     
     r = width + l;
     b = height + t;
-    fl = srcx;
-    fr = srcx + width;
-    ft = srcy;
-    fb = srcy + height;
+    fl = (float)srcx / info->texW[0];
+    fr = (float)(srcx + width) / info->texW[0];
+    ft = (float)srcy / info->texH[0];
+    fb = (float)(srcy + height) / info->texH[0];
 
 #ifdef ACCEL_CP
     BEGIN_RING(24);
